import React, { useState } from "react";
import "./App.scss";

// Spoiler
const Spoiler = ({ header = <h4>+</h4>, open, children }) => {
  const [visible, setVisible] = useState(open);
  return (
    <div className="Spoiler">
      <div className="header" onClick={() => setVisible(!visible)}>
        {header}
      </div>
      <div className={visible ? "visible" : "hidden"}>{children}</div>
    </div>
  );
};

// RangeInput

const RangeInput = ({ min = 1, max = 10 }) => {
  const [len, setLen] = useState(min);
  return (
    <div className="RangeInput">
      <label htmlFor="myInput">
        {len < min ? `The length should be min: ${min}` : ""}
        {len > max ? `The length should be max: ${max}` : ""}
      </label>
      <input
        style={{ borderColor: len < min || len > max ? "red" : "" }}
        type="text"
        onInput={(e) => setLen(e.target.value.length)}
        placeholder="Type your text"
        id="myInput"
      />
    </div>
  );
};

// PasswordConfirm

const PasswordConfirm = ({ min = 4 }) => {
  const [len, setLen] = useState(min);
  const [firstPass, setFirstPass] = useState("");
  const [secondPass, setSecondPass] = useState("");
  return (
    <div className="PasswordConfirm">
      <label htmlFor="passFirst">
        {len < min
          ? `The length should be min: ${min}`
          : !/[A-Z]/g.test(firstPass)
          ? "The password must contain at least one capital letter"
          : "Enter your password"}
      </label>
      <input
        style={{ borderColor: len < min ? "red" : "" }}
        type="password"
        onInput={(e) => setLen(e.target.value.length)}
        onChange={(e) => setFirstPass(e.target.value)}
        id="passFirst"
      />
      <label htmlFor="passSecond">
        {firstPass !== secondPass
          ? "Passwords must match"
          : "Repeat your password"}
      </label>
      <input
        style={{
          borderColor: len < min || firstPass !== secondPass ? "red" : "",
        }}
        type="password"
        onInput={(e) => setLen(e.target.value.length)}
        onChange={(e) => setSecondPass(e.target.value)}
        id="passSecond"
      />
    </div>
  );
};

function App() {
  return (
    <div className="App">
      <Spoiler header={<h1>Заголовок</h1>} open>
        <h2>Контент 1</h2>
        <p>
          Lorem ipsum dolor sit amet consectetur adipisicing elit. Laudantium
          eveniet pariatur ut debitis harum modi. Nesciunt magnam, reiciendis
          ipsa soluta reprehenderit iste necessitatibus fugiat non obcaecati
          officia delectus asperiores eaque numquam quae rem esse neque. Dolorum
          provident doloribus voluptate neque ea nostrum maxime enim. Magni
          debitis saepe nulla pariatur consequuntur.
        </p>
      </Spoiler>
      <Spoiler>
        <h2>Контент 2</h2>
        <p>
          Lorem ipsum, dolor sit amet consectetur adipisicing elit. Amet quo
          enim iusto quibusdam? Maxime amet asperiores numquam, eaque,
          reprehenderit hic provident, a praesentium iure aperiam sapiente?
          Earum aperiam sed odio facere, optio dignissimos debitis provident
          pariatur expedita iure obcaecati ut alias beatae, delectus
          accusantium, atque animi numquam. Aspernatur, placeat beatae!
        </p>
      </Spoiler>

      <RangeInput min={2} max={10} />

      <PasswordConfirm min={4} />
    </div>
  );
}

export default App;
